#ifndef ILLUMESTIMATORS_CREATE_PER_SEGMENT_ESTIMATES_COMMAND_H
#define ILLUMESTIMATORS_CREATE_PER_SEGMENT_ESTIMATES_COMMAND_H

#include "command.h"
#include "commands/create_per_segment_estimates_config.h"
#include "superpixelsegmentation.h"
#include <opencv2/imgproc/imgproc.hpp>
#include "illuminantestimator.h"

namespace illumestimators {

class CommandCreatePerSegmentEstimates : public vole::Command
{
public:
	CommandCreatePerSegmentEstimates();
	int execute();

	void printShortHelp() const;
	void printHelp() const;

	CreatePerSegmentEstimatesConfig config;

	void getSuperpixelGroundTruth(
		cv::Mat_<cv::Vec3d> &ground_truth, std::vector<superpixels::Superpixel> &segmentation,
		std::vector<cv::Vec3d> &gt_values, cv::Mat_<cv::Vec3d> &sp_gt_image);

	bool save(std::string filename);
	bool load(std::string filename);

	int keyToNum(std::string &key);

	IlluminantEstimator *getGamut2D(std::string db_file);
	IlluminantEstimator *getGamut3D(std::string db_file);
	IlluminantEstimator *getBayesian(std::string db_file);
	IlluminantEstimator *getGw(int n, int p, int s);
	void get_training_and_testing_for_set(int set,
		std::vector<std::vector<cv::Vec3d> > &trainingEstimates, std::vector<cv::Vec3d> &trainingIlluminants,
		std::vector<std::vector<cv::Vec3d> > &testingEstimates, std::vector<cv::Vec3d> &gtIlluminants);

protected:
	void compute_illuminant_error(std::vector<cv::Vec3d> &illuminants, std::vector<cv::Vec3d> &gt_values, cv::Mat_<double> &errors);

	// counts the number of different scenes in the "results" vector
	int countScenes();

	std::vector<std::vector<double> > results; 

private:


};


}

#endif // ILLUMESTIMATORS_CREATE_PER_SEGMENT_ESTIMATES_COMMAND_H
