#include "gamut2dconfig.h"
#include <gamutmapping/gamutmapping2destimator.h>

namespace illumestimators {

Gamut2DConfig::Gamut2DConfig(const std::string& prefix) :
	Config(prefix)
{
	#ifdef WITH_BOOST
		initBoostOptions();
	#endif // WITH_BOOST
}

std::string Gamut2DConfig::getString() const
{
	std::stringstream s;

	if (prefix_enabled) {
		s << "[" << prefix << "]" << std::endl;
	} else {
		s << "input=" << inputFile << " # Image to process" << std::endl;
		//s << "output=" << outputDirectory << " # Working directory" << std::endl;
	}

	return s.str();
}

#ifdef WITH_BOOST
void Gamut2DConfig::initBoostOptions()
{
	if (!prefix_enabled) {
		options.add_options()(key("input,I"), boost::program_options::value(&inputFile)->default_value(""), "Image to process");
		//options.add_options()(key("output,O"), value(&outputDirectory)->default_value("/tmp/"), "Working directory");
	}

	std::stringstream description;
	description << "Method: " << GamutMapping2DEstimator::MAX << " = max, " << GamutMapping2DEstimator::MEAN << " = mean, " << GamutMapping2DEstimator::MEDIAN << " = median";

	options.add_options()(key("method,m"), boost::program_options::value(&method)->default_value(GamutMapping2DEstimator::MAX), description.str().c_str());
	options.add_options()(key("n"), boost::program_options::value(&n)->default_value(0), "Derivative order");
	options.add_options()(key("sigma"), boost::program_options::value(&sigma)->default_value(0), "Standard deviation of Gaussian smoothing filter");
	options.add_options()(key("train,T"), boost::program_options::value(&trainingFiles)->multitoken(), "List of training files");

	options.add_options()(key("meta,M"), boost::program_options::value(&metaFiles)->multitoken(), "List of meta files");
	options.add_options()(key("load,l"), boost::program_options::value(&loadFile)->default_value(""), "Load illuminant estimator from file");
	options.add_options()(key("save,s"), boost::program_options::value(&saveFile)->default_value(""), "Save illuminant estimator to file");
}
#endif // WITH_BOOST

} // namespace illumestimators
