#ifndef ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING2DESTIMATOR_H
#define ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING2DESTIMATOR_H

#include "illuminantestimator.h"
#include "superpixelsegmentation.h"

namespace illumestimators {

class GamutMapping2DEstimator : public IlluminantEstimator
{
public:
	enum Method { MAX, MEAN, MEDIAN };

public:
	GamutMapping2DEstimator(Method method = GamutMapping2DEstimator::MAX, int n = 0, double sigma = 0, int verbosity = 0, bool cache = false);
	GamutMapping2DEstimator(const std::vector<cv::Point2f>& canonical, Method method, int n, double sigma, int verbosity = 0, bool cache = false);
	~GamutMapping2DEstimator();

public:
	std::string name() const;
	std::string identifier() const;

public:
	void setCanonicalGamut(const std::vector<cv::Point2f>& canonical);
	void setDerivativeOrder(int n) { m_n = n; }
	void setSigma(double sigma) { m_sigma = sigma; }

public:
	std::vector<cv::Point2f> getCanonicalGamut() { return m_canonical; }
	int getDerivativeOrder() { return m_n; }
	double getSigma() { return m_sigma; }

public:
	using IlluminantEstimator::estimateIlluminant;
	Illum estimateIlluminant(const cv::Mat_<cv::Vec3d>& image, const cv::Mat_<unsigned char>& mask) const;
	Illum estimateIlluminant(const cv::Mat_<cv::Vec3d>& image, const superpixels::Superpixel &superpixel, const cv::Mat_<unsigned char>& mask) const;

	// EXTENSION FOR CPCV
	void estimateIlluminants(
		const cv::Mat_<cv::Vec3d>& image,
		const std::vector<superpixels::Superpixel>& superpixels,
		std::vector<Illum> &estimated_illuminants,
		const cv::Mat_<unsigned char>& mask) const;

public:
	bool train(const std::vector<std::string>& imageFiles, const std::vector<std::string>& colorspaces, const std::vector<cv::Vec3d>& illuminants, const std::vector<std::string>& maskFiles);
	bool train(const std::vector<std::vector<cv::Point2f> >& gamuts, const std::vector<cv::Vec3d>& illuminants);


	// EXTENSIONS FOR CPCV
	bool addTrainingData(
		cv::Mat_<cv::Vec3d> &img,
		const std::vector<superpixels::Superpixel>& superpixels,
		const std::vector<cv::Vec3d>& illuminants,
		cv::Mat_<unsigned char> &mask
	);
	bool finalizeTraining();
	std::vector<cv::Point2f> calculateCanonicalGamutSuperpixel(
		cv::Mat_<cv::Vec3d> &img,
		const superpixels::Superpixel &superpixel,
		const cv::Vec3d& illuminant,
		const cv::Mat_<unsigned char>& mask
	);

public:
	bool save(const std::string& filename) const;
	bool load(const std::string& filename);
	virtual int error(); // dummy implementation, returns always 0

public:
	std::vector<cv::Point2f> calculateCanonicalGamut(const std::string& filename, const std::string& colorspace, const cv::Vec3d& illuminant, const cv::Mat_<unsigned char>& mask, bool preprocess = false);
	std::vector<cv::Point2f> calculateImageGamut(const std::vector<cv::Point2f>& chromaticities) const;
	std::vector<cv::Point2f> calculatePossibleMappings(const std::vector<cv::Point2f>& imageGamut, const std::vector<cv::Point2f>& canonicalGamut) const;
	std::vector<cv::Point2f> calculateSetOfPossibleIlluminants(const std::vector<cv::Vec3d>& illuminants) const;
	std::vector<cv::Vec3d> calculateMonteCarloIntersection(const std::vector<cv::Point2f>& possibleMappings, const std::vector<cv::Point2f>& possibleIlluminants) const;
	Illum findBestGamut(std::vector<cv::Vec3d>& randomVectors) const;

	// CHANGED VISIBILITY TO PUBLIC FOR CPCV
	void preprocessImage(const cv::Mat_<cv::Vec3d>& image, const cv::Mat_<unsigned char>& mask, cv::Mat_<cv::Vec3d>& outputImage, cv::Mat_<unsigned char>& outputMask) const;

private:
	double sumlog(const std::vector<double>& values) const;

private:
	Method m_method;
	int m_n;
	double m_sigma;
	std::vector<cv::Point2f> m_canonical;
	std::vector<cv::Vec3d> m_illuminants;
	int m_verbosity;
	bool m_cache;
};

} // namespace illumestimators

#endif // ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING2DESTIMATOR_H
