#ifndef ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING3DESTIMATOR_H
#define ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING3DESTIMATOR_H

#include "illuminantestimator.h"
#include "common/convexhull.h"

namespace illumestimators {

class GamutMapping3DEstimator : public IlluminantEstimator
{
public:
	GamutMapping3DEstimator(int n = 0, double sigma = 0, int verbosity = 0, bool cache = false);
	GamutMapping3DEstimator(const std::vector<cv::Vec3d>& canonical, int n, double sigma, int verbosity = 0, bool cache = false);
	~GamutMapping3DEstimator();

public:
	std::string name() const;
	std::string identifier() const;

public:
	void setCanonicalGamut(const std::vector<cv::Vec3d>& canonical);
	void setDerivativeOrder(int n) { m_n = n; }
	void setSigma(double sigma) { m_sigma = sigma; }

public:
	std::vector<cv::Vec3d> getCanonicalGamut() { return m_canonical.vertices(); }
	int getDerivativeOrder() { return m_n; }
	double getSigma() { return m_sigma; }

public:
	using IlluminantEstimator::estimateIlluminant;
	Illum estimateIlluminant(const cv::Mat_<cv::Vec3d>& image, const cv::Mat_<unsigned char>& mask) const;
	Illum estimateIlluminant(const cv::Mat_<cv::Vec3d>& image, const superpixels::Superpixel &superpixel, const cv::Mat_<unsigned char>& mask) const;

public:
	bool train(const std::vector<std::string>& imageFiles, const std::vector<std::string>& colorspaces, const std::vector<cv::Vec3d>& illuminants, const std::vector<std::string>& maskFiles = std::vector<std::string>());
	bool train(const std::vector<std::vector<cv::Vec3d> >& gamuts);

public:
	bool save(const std::string& filename) const;
	bool load(const std::string& filename);
	virtual int error(); // dummy implementation, returns always 0

public:
	std::vector<cv::Vec3d> calculateCanonicalGamut(const std::string& filename, const std::string& colorspace, const cv::Vec3d& illuminant, const cv::Mat_<unsigned char>& mask);

	virtual void preprocessImage(const cv::Mat_<cv::Vec3d>& image, const cv::Mat_<unsigned char>& mask, cv::Mat_<cv::Vec3d> &outputImage, cv::Mat_<unsigned char> &outputMask) const;
private:
	Illum calculateMapping(const std::vector<cv::Vec3d>& vertices) const;

private:
	ConvexHull<3> m_canonical;
	std::vector<Facet<3> > m_facets;
	std::vector<cv::Vec3d> m_vertices;
	int m_n;
	double m_sigma;
	int verbosity;
	bool m_cache;
};

} // namespace illumestimators

#endif // ILLUMESTIMATORS_GAMUTMAPPING_GAMUTMAPPING3DESTIMATOR_H
